#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#define EOD -1
#define CELL_SIZE 48

void main(void)
{
    char *inname="../tcp.txt";
    char *outname1 ="clr.dat";
    char *outname2 = "plr.dat";

    FILE *infile, *outfile1, *outfile2;

    int host, size;
    double time;
    int maxbuf;

#ifndef NDEBUG
    const int Buffer[]={400,EOD};
    const int Capacity[]={12200,12600,EOD};
#else
    const int Buffer[]={100,400,1600,6400,25600,100000,EOD};
    const int Capacity[]={500,1000,2000,4000,8000,16000,EOD};
#endif

    int Buffer_index = 0;
    int Capacity_index = 0;

    int B;
    int C;
    
    infile = fopen(inname,"rt");
    if (!infile)
    {
        printf("Cannot open file\n");
        return 1;
    }
    
    outfile1 = fopen(outname1,"wt");
    if (!outfile1)
    {
        printf("Cannot open file\n");
        return 1;
    }

    outfile2 = fopen(outname2,"wt");
    if (!outfile2)
    {
        printf("Cannot open file\n");
        return 1;
    }
    
    while ( (B = Buffer[Buffer_index]) != EOD) {

         //open the file
         char name[32];
         FILE* outfile3;

         sprintf(name,"%d.dat",B);
         outfile3 = fopen(name,"wt");
         if (!outfile3)
         {
            printf("Cannot open file %s\n",name);
            return 1;
         }

      maxbuf=0;
      while ( (C = Capacity[Capacity_index]) != EOD) {
         
         double leak_rate = (double) 1/C;
         double next_tip = (double) 0;

         int free_buffer = B;
         int cln = 0;
         int pln = 0;

         int total_cell = 0;
         int total_packet = 0;
         int num_out;

         while (fscanf(infile,"%lf%d%d\n",&time,&host,&size)==3)
         {
            int cell_size = size / CELL_SIZE;

            if (size % CELL_SIZE != 0)
               cell_size++;

            total_cell += cell_size;
            total_packet++;
           
            
            if (time>next_tip)
            {
              num_out = int((time-next_tip)/leak_rate);
              free_buffer+=num_out;
              if(free_buffer > B) //buffer underflow
                free_buffer = B;
              next_tip += leak_rate*num_out;
            }
            
            /*
            while(time >= next_tip) { //remove cells from the buffer
               free_buffer++;
               if(free_buffer > B) //buffer underflow
                  free_buffer = B;
               next_tip += leak_rate;
             }
             */

             //add new cells into the switch
             free_buffer = free_buffer - cell_size;

             if (B-free_buffer>maxbuf)
               maxbuf = B-free_buffer;

             if (free_buffer < 0) { //buffer overflow
                cln += abs(free_buffer);
                pln ++;
                free_buffer = 0;
             }
         } //end of while

         rewind(infile);

         double clr = (double)cln/total_cell;
         double plr = (double)pln/total_packet;

         fprintf(outfile3,"%d\t%.8f\n",C,clr); //output to file "Buffer_size"

         fprintf(outfile1,"%.8f\t",clr); //output to clr table
         fprintf(outfile2,"%.8f\t",plr); //output to plr table

         Capacity_index++;

         printf("%d\t%d\t%.8f\t%.8f\tMaxBuf:%d\n",B,C,clr,plr,maxbuf);

      } // while capacity

      Buffer_index++;
      Capacity_index = 0;

      fprintf(outfile1,"\n");
      fprintf(outfile2,"\n");

      fclose(outfile3);

    } // while buffer size
    
    fclose(infile);
    fclose(outfile1);
    fclose(outfile2);
    printf("%d sec run time\n", clock()/CLOCKS_PER_SEC);
}
