#include <stdlib.h>
#include <math.h>
#include <stdio.h>

#define PI 3.14159265358979
#define MAX_SAMPLES	1000000

int N;
double *samples;

/***************************************************************\
|************************** F_HAT ******************************|
     input: lambda
     output: f_hat of lambda, as defined 
\***************************************************************/
double periodogram(double lambda_L)
{
    int i;
    double sum   = 0.0;
    double s_sum = 0.0;
    double c_sum = 0.0;

    for (i=0;i<N;i++)
     {
        c_sum += samples[i] * cos( (double) i * lambda_L);
        s_sum += samples[i] * sin( (double) i * lambda_L);
     }
    sum = c_sum * c_sum + s_sum * s_sum;
    return (sum / (2.0 * PI * (double) N));
}


/***************************************************************\
|************************** F_HAT ******************************|
     input: lambda
     output: f_hat of lambda, as defined 
\***************************************************************/
double f_hat(double lambda_M)
{
    int i,n;
    double sum = 0.0;

    n=(int) ((double) N * lambda_M / (2.0 * PI));
    for (i=1;i<=n; i++)
     {
      if ( (i % 1000) == 0)
       printf("Processed %d samples\n", i);
      sum += periodogram( (double) i * 2.0 * PI / (double) N);
     }

    return (sum * 2.0 * PI / (double) N);
}
  

/***************************************************************\
|***************************************************************|
|********************** MAIN THING *****************************|
|***************************************************************|
\***************************************************************/
int main()
{
    double bits;
    double temp, temp1;

    double Q = 0.5;
    double lambda_M = 0.5;
    FILE *infile;

    samples=(double *) malloc (sizeof(double) * MAX_SAMPLES);

    N=0;
    
   if ((infile = fopen("bytesper0.1sec.dat","r")) == NULL)
   {
      printf("\nUnable to open input file.\n");
      exit(1);
   }

   printf("open the file\n");

    while ( fscanf(infile, "%lf", &bits) != EOF ) {
	    samples[N] = bits;
	    N++;
    }
    
    printf("There are %d samples read\n", N);
    
    temp = f_hat( Q * lambda_M);
    temp /= f_hat( lambda_M);
    temp1 = log(temp)/ (2.0 * log(Q));
    printf("%lf\t%lf\n",temp,temp1);
    printf("And the Hurst estimate is %8.4f\n", 1 - temp1);

    return 0;
}
  


